@@ -45,7 +45,6 @@
 
 #include "ares.h"
 #include "ares_inet_net_pton.h"
-#include "bitncmp.h"
 #include "ares_platform.h"
 #include "ares_private.h"
 
@@ -120,6 +119,8 @@ void ares_gethostbyname(ares_channel_t *channel, const char *name, int family,
   ghbn_arg->arg      = arg;
   ghbn_arg->channel  = channel;
 
+  /* NOTE: ares_getaddrinfo() locks the channel, we don't use the channel
+   *       outside so no need to lock */
   ares_getaddrinfo(channel, name, NULL, &hints, ares_gethostbyname_callback,
                    ghbn_arg);
 }
@@ -160,24 +161,23 @@ static void sort_addresses(const struct hostent  *host,
 static size_t get_address_index(const struct in_addr  *addr,
                                 const struct apattern *sortlist, size_t nsort)
 {
-  size_t i;
+  size_t           i;
+  struct ares_addr aaddr;
+
+  memset(&aaddr, 0, sizeof(aaddr));
+  aaddr.family = AF_INET;
+  memcpy(&aaddr.addr.addr4, addr, 4);
 
   for (i = 0; i < nsort; i++) {
-    if (sortlist[i].family != AF_INET) {
+    if (sortlist[i].addr.family != AF_INET) {
       continue;
     }
-    if (sortlist[i].type == PATTERN_MASK) {
-      if ((addr->s_addr & sortlist[i].mask.addr4.s_addr) ==
-          sortlist[i].addr.addr4.s_addr) {
-        break;
-      }
-    } else {
-      if (!ares__bitncmp(&addr->s_addr, &sortlist[i].addr.addr4.s_addr,
-                         sortlist[i].mask.bits)) {
-        break;
-      }
+
+    if (ares__subnet_match(&aaddr, &sortlist[i].addr, sortlist[i].mask)) {
+      break;
     }
   }
+
   return i;
 }
 
@@ -217,13 +217,19 @@ static void sort6_addresses(const struct hostent  *host,
 static size_t get6_address_index(const struct ares_in6_addr *addr,
                                  const struct apattern *sortlist, size_t nsort)
 {
-  size_t i;
+  size_t           i;
+  struct ares_addr aaddr;
+
+  memset(&aaddr, 0, sizeof(aaddr));
+  aaddr.family = AF_INET6;
+  memcpy(&aaddr.addr.addr6, addr, 16);
 
   for (i = 0; i < nsort; i++) {
-    if (sortlist[i].family != AF_INET6) {
+    if (sortlist[i].addr.family != AF_INET6) {
       continue;
     }
-    if (!ares__bitncmp(addr, &sortlist[i].addr.addr6, sortlist[i].mask.bits)) {
+
+    if (ares__subnet_match(&aaddr, &sortlist[i].addr, sortlist[i].mask)) {
       break;
     }
   }
@@ -263,8 +269,9 @@ done:
 
 /* I really have no idea why this is exposed as a public function, but since
  * it is, we can't kill this legacy function. */
-int ares_gethostbyname_file(ares_channel_t *channel, const char *name,
-                            int family, struct hostent **host)
+static ares_status_t ares_gethostbyname_file_int(ares_channel_t *channel,
+                                                 const char *name, int family,
+                                                 struct hostent **host)
 {
   const ares_hosts_entry_t *entry;
   ares_status_t             status;
@@ -302,8 +309,22 @@ done:
    * as permissions errors reading /etc/hosts or a malformed /etc/hosts */
   if (status != ARES_SUCCESS && status != ARES_ENOMEM &&
       ares__is_localhost(name)) {
-    return (int)ares__hostent_localhost(name, family, host);
+    return ares__hostent_localhost(name, family, host);
   }
 
+  return status;
+}
+
+int ares_gethostbyname_file(ares_channel_t *channel, const char *name,
+                            int family, struct hostent **host)
+{
+  ares_status_t status;
+  if (channel == NULL) {
+    return ARES_ENOTFOUND;
+  }
+
+  ares__channel_lock(channel);
+  status = ares_gethostbyname_file_int(channel, name, family, host);
+  ares__channel_unlock(channel);
   return (int)status;
 }
