@@ -28,7 +28,7 @@
 #ifdef HAVE_GETSERVBYPORT_R
 #  if !defined(GETSERVBYPORT_R_ARGS) || (GETSERVBYPORT_R_ARGS < 4) || \
     (GETSERVBYPORT_R_ARGS > 6)
-#    error "you MUST specifiy a valid number of arguments for getservbyport_r"
+#    error "you MUST specify a valid number of arguments for getservbyport_r"
 #  endif
 #endif
 
@@ -47,7 +47,7 @@
 #ifdef HAVE_NET_IF_H
 #  include <net/if.h>
 #endif
-#ifdef HAVE_IPHLPAPI_H
+#if defined(USE_WINSOCK) && defined(HAVE_IPHLPAPI_H)
 #  include <iphlpapi.h>
 #endif
 
@@ -69,7 +69,7 @@ struct nameinfo_query {
   size_t       timeouts;
 };
 
-#ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 #  define IPBUFSIZ \
     (sizeof("ffff:ffff:ffff:ffff:ffff:ffff:255.255.255.255") + IF_NAMESIZE)
 #else
@@ -80,15 +80,16 @@ static void  nameinfo_callback(void *arg, int status, int timeouts,
                                struct hostent *host);
 static char *lookup_service(unsigned short port, unsigned int flags, char *buf,
                             size_t buflen);
-#ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 static void append_scopeid(const struct sockaddr_in6 *addr6,
                            unsigned int scopeid, char *buf, size_t buflen);
 #endif
-STATIC_TESTABLE char *ares_striendstr(const char *s1, const char *s2);
+static char *ares_striendstr(const char *s1, const char *s2);
 
-void ares_getnameinfo(ares_channel_t *channel, const struct sockaddr *sa,
-                      ares_socklen_t salen, int flags_int,
-                      ares_nameinfo_callback callback, void *arg)
+static void  ares_getnameinfo_int(ares_channel_t        *channel,
+                                  const struct sockaddr *sa,
+                                  ares_socklen_t salen, int flags_int,
+                                  ares_nameinfo_callback callback, void *arg)
 {
   const struct sockaddr_in  *addr  = NULL;
   const struct sockaddr_in6 *addr6 = NULL;
@@ -144,7 +145,7 @@ void ares_getnameinfo(ares_channel_t *channel, const struct sockaddr *sa,
       if (salen == sizeof(struct sockaddr_in6)) {
         ares_inet_ntop(AF_INET6, &addr6->sin6_addr, ipbuf, IPBUFSIZ);
         /* If the system supports scope IDs, use it */
-#ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
         append_scopeid(addr6, flags, ipbuf, sizeof(ipbuf));
 #endif
       } else {
@@ -185,6 +186,19 @@ void ares_getnameinfo(ares_channel_t *channel, const struct sockaddr *sa,
   }
 }
 
+void ares_getnameinfo(ares_channel_t *channel, const struct sockaddr *sa,
+                      ares_socklen_t salen, int flags_int,
+                      ares_nameinfo_callback callback, void *arg)
+{
+  if (channel == NULL) {
+    return;
+  }
+
+  ares__channel_lock(channel);
+  ares_getnameinfo_int(channel, sa, salen, flags_int, callback, arg);
+  ares__channel_unlock(channel);
+}
+
 static void nameinfo_callback(void *arg, int status, int timeouts,
                               struct hostent *host)
 {
@@ -233,7 +247,7 @@ static void nameinfo_callback(void *arg, int status, int timeouts,
       ares_inet_ntop(AF_INET, &niquery->addr.addr4.sin_addr, ipbuf, IPBUFSIZ);
     } else {
       ares_inet_ntop(AF_INET6, &niquery->addr.addr6.sin6_addr, ipbuf, IPBUFSIZ);
-#ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
       append_scopeid(&niquery->addr.addr6, niquery->flags, ipbuf,
                      sizeof(ipbuf));
 #endif
@@ -332,7 +346,7 @@ static char *lookup_service(unsigned short port, unsigned int flags, char *buf,
   return NULL;
 }
 
-#ifdef HAVE_SOCKADDR_IN6_SIN6_SCOPE_ID
+#ifdef HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID
 static void append_scopeid(const struct sockaddr_in6 *addr6, unsigned int flags,
                            char *buf, size_t buflen)
 {
@@ -373,7 +387,7 @@ static void append_scopeid(const struct sockaddr_in6 *addr6, unsigned int flags,
 #endif
 
 /* Determines if s1 ends with the string in s2 (case-insensitive) */
-STATIC_TESTABLE char *ares_striendstr(const char *s1, const char *s2)
+static char *ares_striendstr(const char *s1, const char *s2)
 {
   const char *c1;
   const char *c2;
