@@ -54,9 +54,11 @@ ares_status_t ares_send_ex(ares_channel_t *channel, const unsigned char *qbuf,
 {
   struct query  *query;
   size_t         packetsz;
-  struct timeval now;
+  struct timeval now = ares__tvnow();
   ares_status_t  status;
-  unsigned short id = generate_unique_qid(channel);
+  unsigned short id   = generate_unique_qid(channel);
+  unsigned char *abuf = NULL;
+  size_t         alen = 0;
 
   /* Verify that the query is at least long enough to hold the header. */
   if (qlen < HFIXEDSZ || qlen >= (1 << 16)) {
@@ -64,9 +66,20 @@ ares_status_t ares_send_ex(ares_channel_t *channel, const unsigned char *qbuf,
     return ARES_EBADQUERY;
   }
   if (ares__slist_len(channel->servers) == 0) {
-    callback(arg, ARES_ESERVFAIL, 0, NULL, 0);
-    return ARES_ESERVFAIL;
+    callback(arg, ARES_ENOSERVER, 0, NULL, 0);
+    return ARES_ENOSERVER;
   }
+
+  /* Check query cache */
+  status = ares_qcache_fetch(channel, &now, qbuf, qlen, &abuf, &alen);
+  if (status != ARES_ENOTFOUND) {
+    /* ARES_SUCCESS means we retrieved the cache, anything else is a critical
+     * failure, all result in termination */
+    callback(arg, (int)status, 0, abuf, (int)alen);
+    ares_free(abuf);
+    return status;
+  }
+
   /* Allocate space for query and allocated fields. */
   query = ares_malloc(sizeof(struct query));
   if (!query) {
@@ -129,7 +142,6 @@ ares_status_t ares_send_ex(ares_channel_t *channel, const unsigned char *qbuf,
   }
 
   /* Perform the first query action. */
-  now = ares__tvnow();
 
   status = ares__send_query(query, &now);
   if (status == ARES_SUCCESS && qid) {
@@ -141,5 +153,30 @@ ares_status_t ares_send_ex(ares_channel_t *channel, const unsigned char *qbuf,
 void ares_send(ares_channel_t *channel, const unsigned char *qbuf, int qlen,
                ares_callback callback, void *arg)
 {
+  if (channel == NULL) {
+    return;
+  }
+
+  ares__channel_lock(channel);
+
   ares_send_ex(channel, qbuf, (size_t)qlen, callback, arg, NULL);
+
+  ares__channel_unlock(channel);
+}
+
+size_t ares_queue_active_queries(ares_channel_t *channel)
+{
+  size_t len;
+
+  if (channel == NULL) {
+    return 0;
+  }
+
+  ares__channel_lock(channel);
+
+  len = ares__llist_len(channel->all_queries);
+
+  ares__channel_unlock(channel);
+
+  return len;
 }
