@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -705,6 +713,37 @@ CONTAINED_TEST_F(LibraryTest, ContainerEmptyInit,
   return HasFailure();
 }
 
+// Test that init fails if the flag to not use a default local named server is
+// enabled and no other nameservers are available.
+CONTAINED_TEST_F(LibraryTest, ContainerNoDfltSvrEmptyInit,
+                 "myhostname", "mydomainname.org", empty) {
+  ares_channel_t *channel = nullptr;
+  struct ares_options opts = {0};
+  int optmask = ARES_OPT_FLAGS;
+  opts.flags = ARES_FLAG_NO_DFLT_SVR;
+  EXPECT_EQ(ARES_ENOSERVER, ares_init_options(&channel, &opts, optmask));
+
+  EXPECT_EQ(nullptr, channel);
+  return HasFailure();
+}
+// Test that init succeeds if the flag to not use a default local named server
+// is enabled but other nameservers are available.
+CONTAINED_TEST_F(LibraryTest, ContainerNoDfltSvrFullInit,
+                 "myhostname", "mydomainname.org", filelist) {
+  ares_channel_t *channel = nullptr;
+  struct ares_options opts = {0};
+  int optmask = ARES_OPT_FLAGS;
+  opts.flags = ARES_FLAG_NO_DFLT_SVR;
+  EXPECT_EQ(ARES_SUCCESS, ares_init_options(&channel, &opts, optmask));
+
+  std::string actual = GetNameServers(channel);
+  std::string expected = "1.2.3.4:53";
+  EXPECT_EQ(expected, actual);
+
+  ares_destroy(channel);
+  return HasFailure();
+}
+
 #endif
 
 }  // namespace test
