@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -36,7 +44,7 @@ TEST_F(LibraryTest, ParseTxtReplyOK) {
   std::vector<byte> data = pkt.data();
 
   struct ares_txt_reply* txt = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   ASSERT_NE(nullptr, txt);
   EXPECT_EQ(std::vector<byte>(expected1.data(), expected1.data() + expected1.size()),
             std::vector<byte>(txt->txt, txt->txt + txt->length));
@@ -63,7 +71,7 @@ TEST_F(LibraryTest, ParseTxtExtReplyOK) {
   std::vector<byte> data = pkt.data();
 
   struct ares_txt_ext* txt = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_txt_reply_ext(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_txt_reply_ext(data.data(), (int)data.size(), &txt));
   ASSERT_NE(nullptr, txt);
   EXPECT_EQ(std::vector<byte>(expected1.data(), expected1.data() + expected1.size()),
             std::vector<byte>(txt->txt, txt->txt + txt->length));
@@ -107,7 +115,7 @@ TEST_F(LibraryTest, ParseTxtMalformedReply1) {
   };
 
   struct ares_txt_reply* txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   ASSERT_EQ(nullptr, txt);
 }
 
@@ -135,7 +143,7 @@ TEST_F(LibraryTest, ParseTxtMalformedReply2) {
   };
 
   struct ares_txt_reply* txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   ASSERT_EQ(nullptr, txt);
 }
 
@@ -166,7 +174,7 @@ TEST_F(LibraryTest, ParseTxtMalformedReply3) {
   };
 
   struct ares_txt_reply* txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   ASSERT_EQ(nullptr, txt);
 }
 
@@ -188,7 +196,7 @@ TEST_F(LibraryTest, ParseTxtMalformedReply4) {
   };
 
   struct ares_txt_reply* txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   ASSERT_EQ(nullptr, txt);
 }
 
@@ -209,7 +217,7 @@ TEST_F(LibraryTest, ParseTxtReplyErrors) {
   pkt.questions_.clear();
   data = pkt.data();
   txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   EXPECT_EQ(nullptr, txt);
   pkt.add_question(new DNSQuestion("example.com", T_MX));
 
@@ -218,7 +226,7 @@ TEST_F(LibraryTest, ParseTxtReplyErrors) {
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("Axample.com", T_TXT));
   data = pkt.data();
-  EXPECT_EQ(ARES_ENODATA, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_ENODATA, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("example.com", T_TXT));
 #endif
@@ -227,7 +235,7 @@ TEST_F(LibraryTest, ParseTxtReplyErrors) {
   pkt.add_question(new DNSQuestion("example.com", T_MX));
   data = pkt.data();
   txt = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   EXPECT_EQ(nullptr, txt);
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("example.com", T_MX));
@@ -236,14 +244,14 @@ TEST_F(LibraryTest, ParseTxtReplyErrors) {
   pkt.answers_.clear();
   data = pkt.data();
   txt = nullptr;
-  EXPECT_EQ(ARES_ENODATA, ares_parse_txt_reply(data.data(), data.size(), &txt));
+  EXPECT_EQ(ARES_ENODATA, ares_parse_txt_reply(data.data(), (int)data.size(), &txt));
   EXPECT_EQ(nullptr, txt);
   pkt.add_answer(new DNSTxtRR("example.com", 100, {expected1}));
 
   // Truncated packets.
   for (size_t len = 1; len < data.size(); len++) {
     txt = nullptr;
-    EXPECT_NE(ARES_SUCCESS, ares_parse_txt_reply(data.data(), len, &txt));
+    EXPECT_NE(ARES_SUCCESS, ares_parse_txt_reply(data.data(), (int)len, &txt));
     EXPECT_EQ(nullptr, txt);
   }
 }
@@ -265,7 +273,7 @@ TEST_F(LibraryTest, ParseTxtReplyAllocFail) {
   for (int ii = 1; ii <= 13; ii++) {
     ClearFails();
     SetAllocFail(ii);
-    EXPECT_EQ(ARES_ENOMEM, ares_parse_txt_reply(data.data(), data.size(), &txt)) << ii;
+    EXPECT_EQ(ARES_ENOMEM, ares_parse_txt_reply(data.data(), (int)data.size(), &txt)) << ii;
   }
 }
 
