@@ -1,17 +1,25 @@
-/*
- * Copyright (C) The c-ares project
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -33,7 +41,7 @@ TEST_F(LibraryTest, ParseUriReplyOK) {
   std::vector<byte> data = pkt.data();
 
   struct ares_uri_reply* uri = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   ASSERT_NE(nullptr, uri);
 
   EXPECT_EQ("uri.example.com", std::string(uri->uri));
@@ -67,7 +75,7 @@ TEST_F(LibraryTest, ParseUriReplySingle) {
   std::vector<byte> data = pkt.data();
 
   struct ares_uri_reply* uri = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   ASSERT_NE(nullptr, uri);
 
   EXPECT_EQ("example.abc.def.com", std::string(uri->uri));
@@ -106,7 +114,7 @@ TEST_F(LibraryTest, ParseUriReplyMalformed) {
   };
 
   struct ares_uri_reply* uri = nullptr;
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   ASSERT_EQ(nullptr, uri);
 }
 
@@ -129,7 +137,7 @@ TEST_F(LibraryTest, ParseUriReplyMultiple) {
   std::vector<byte> data = pkt.data();
 
   struct ares_uri_reply* uri0 = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri0));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri0));
   ASSERT_NE(nullptr, uri0);
   struct ares_uri_reply* uri = uri0;
 
@@ -172,7 +180,7 @@ TEST_F(LibraryTest, ParseUriReplyCname) {
   std::vector<byte> data = pkt.data();
 
   struct ares_uri_reply* uri = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   ASSERT_NE(nullptr, uri);
 
   EXPECT_EQ("uri.abc.def.com", std::string(uri->uri));
@@ -204,7 +212,7 @@ TEST_F(LibraryTest, ParseUriReplyCnameMultiple) {
   std::vector<byte> data = pkt.data();
 
   struct ares_uri_reply* uri0 = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri0));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri0));
   ASSERT_NE(nullptr, uri0);
   struct ares_uri_reply* uri = uri0;
 
@@ -242,7 +250,7 @@ TEST_F(LibraryTest, ParseUriReplyErrors) {
   // No question.
   pkt.questions_.clear();
   data = pkt.data();
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   pkt.add_question(new DNSQuestion("example.abc.def.com", T_URI));
 
 #ifdef DISABLED
@@ -250,7 +258,7 @@ TEST_F(LibraryTest, ParseUriReplyErrors) {
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("Axample.com", T_URI));
   data = pkt.data();
-  EXPECT_EQ(ARES_ENODATA, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_ENODATA, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("example.com", T_URI));
 #endif
@@ -258,7 +266,7 @@ TEST_F(LibraryTest, ParseUriReplyErrors) {
   // Two questions.
   pkt.add_question(new DNSQuestion("example.abc.def.com", T_URI));
   data = pkt.data();
-  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_EBADRESP, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   pkt.questions_.clear();
   pkt.add_question(new DNSQuestion("64.48.32.16.in-addr.arpa", T_PTR));
 
@@ -266,7 +274,7 @@ TEST_F(LibraryTest, ParseUriReplyErrors) {
   pkt.answers_.clear();
   pkt.add_answer(new DNSMxRR("example.com", 100, 100, "mx1.example.com"));
   data = pkt.data();
-  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_SUCCESS, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   EXPECT_EQ(nullptr, uri);
   pkt.answers_.clear();
   pkt.add_answer(new DNSUriRR("example.abc.def.com", 180, 0, 10, "example.abc.def.com"));
@@ -274,13 +282,13 @@ TEST_F(LibraryTest, ParseUriReplyErrors) {
   // No answer.
   pkt.answers_.clear();
   data = pkt.data();
-  EXPECT_EQ(ARES_ENODATA, ares_parse_uri_reply(data.data(), data.size(), &uri));
+  EXPECT_EQ(ARES_ENODATA, ares_parse_uri_reply(data.data(), (int)data.size(), &uri));
   pkt.add_answer(new DNSUriRR("example.abc.def.com", 180, 0, 10, "example.abc.def.com"));
 
   // Truncated packets.
   data = pkt.data();
   for (size_t len = 1; len < data.size(); len++) {
-    int rc = ares_parse_uri_reply(data.data(), len, &uri);
+    int rc = ares_parse_uri_reply(data.data(), (int)len, &uri);
     EXPECT_TRUE(rc == ARES_EBADRESP || rc == ARES_EBADNAME);
   }
 }
@@ -297,7 +305,7 @@ TEST_F(LibraryTest, ParseUriReplyAllocFail) {
   for (int ii = 1; ii <= 5; ii++) {
     ClearFails();
     SetAllocFail(ii);
-    EXPECT_EQ(ARES_ENOMEM, ares_parse_uri_reply(data.data(), data.size(), &uri)) << ii;
+    EXPECT_EQ(ARES_ENOMEM, ares_parse_uri_reply(data.data(), (int)data.size(), &uri)) << ii;
   }
 }
 
