@@ -54,6 +54,7 @@ extern "C" {
 #endif
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <functional>
 #include <sstream>
@@ -92,12 +93,161 @@ const std::vector<std::pair<int, bool>> ipv6_family_both_modes = {
   std::make_pair<int, bool>(AF_INET6, true)
 };
 
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_ipv4_family_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, true),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_ipv6_family_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, true),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int, bool>> all_evsys_both_families_both_modes = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_WIN32, AF_INET6, true),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_KQUEUE, AF_INET6, true),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_EPOLL, AF_INET6, true),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_POLL, AF_INET6, true),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET, true),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, false),
+  std::make_tuple<ares_evsys_t, int, bool>(ARES_EVSYS_SELECT, AF_INET6, true),
+#endif
+};
+
+
+std::vector<std::tuple<ares_evsys_t, int, bool>> evsys_families_modes = all_evsys_both_families_both_modes;
+
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv4_family = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_ipv6_family = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET6),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET6),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET6),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET6),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET6),
+#endif
+};
+
+const std::vector<std::tuple<ares_evsys_t, int>> all_evsys_both_families = {
+#ifdef _WIN32
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_WIN32, AF_INET6),
+#endif
+#ifdef HAVE_KQUEUE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_KQUEUE, AF_INET6),
+#endif
+#ifdef HAVE_EPOLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_EPOLL, AF_INET6),
+#endif
+#ifdef HAVE_POLL
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_POLL, AF_INET6),
+#endif
+#ifdef HAVE_PIPE
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET),
+  std::make_tuple<ares_evsys_t, int>(ARES_EVSYS_SELECT, AF_INET6),
+#endif
+};
+
+
+
+std::vector<std::tuple<ares_evsys_t, int>> evsys_families = all_evsys_both_families;
+
+
 // Which parameters to use in tests
 std::vector<int> families = both_families;
 std::vector<std::pair<int, bool>> families_modes = both_families_both_modes;
 
 unsigned long long LibraryTest::fails_ = 0;
 std::map<size_t, int> LibraryTest::size_fails_;
+std::mutex            LibraryTest::lock_;
 
 void ProcessWork(ares_channel_t *channel,
                  std::function<std::set<ares_socket_t>()> get_extrafds,
@@ -188,33 +338,42 @@ void ProcessWork(ares_channel_t *channel,
   }
 }
 
+
 // static
 void LibraryTest::SetAllocFail(int nth) {
+  lock_.lock();
   assert(nth > 0);
   assert(nth <= (int)(8 * sizeof(fails_)));
   fails_ |= (1LL << (nth - 1));
+  lock_.unlock();
 }
 
 // static
 void LibraryTest::SetAllocSizeFail(size_t size) {
+  lock_.lock();
   size_fails_[size]++;
+  lock_.unlock();
 }
 
 // static
 void LibraryTest::ClearFails() {
+  lock_.lock();
   fails_ = 0;
   size_fails_.clear();
+  lock_.unlock();
 }
 
 
 // static
 bool LibraryTest::ShouldAllocFail(size_t size) {
+  lock_.lock();
   bool fail = (fails_ & 0x01);
   fails_ >>= 1;
   if (size_fails_[size] > 0) {
     size_fails_[size]--;
     fail = true;
   }
+  lock_.unlock();
   return fail;
 }
 
@@ -352,6 +511,14 @@ MockServer::~MockServer() {
   free(tcp_data_);
 }
 
+static unsigned short getaddrport(struct sockaddr_storage *addr)
+{
+  if (addr->ss_family == AF_INET)
+    return ntohs(((struct sockaddr_in *)(void *)addr)->sin_port);
+
+  return ntohs(((struct sockaddr_in6 *)(void *)addr)->sin6_port);
+}
+
 void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr, ares_socklen_t addrlen,
                                byte *data, int len) {
 
@@ -410,7 +577,8 @@ void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr,
   if (verbose) {
     std::vector<byte> req(data, data + len);
     std::cerr << "received " << (fd == udpfd_ ? "UDP" : "TCP") << " request " << PacketToString(req)
-              << " on port " << (fd == udpfd_ ? udpport_ : tcpport_) << std::endl;
+              << " on port " << (fd == udpfd_ ? udpport_ : tcpport_)
+              << ":" << getaddrport(addr) << std::endl;
     std::cerr << "ProcessRequest(" << qid << ", '" << namestr
               << "', " << RRTypeToString(rrtype) << ")" << std::endl;
   }
@@ -482,6 +650,7 @@ std::set<ares_socket_t> MockServer::fds() const {
   return result;
 }
 
+
 void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr, ares_socklen_t addrlen,
                                 int qid, const std::string& name, int rrtype) {
   // Before processing, let gMock know the request is happening.
@@ -503,8 +672,11 @@ void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr,
     reply[0] = (byte)((qid >> 8) & 0xff);
     reply[1] = (byte)(qid & 0xff);
   }
-  if (verbose) std::cerr << "sending reply " << PacketToString(reply)
-                         << " on port " << ((fd == udpfd_) ? udpport_ : tcpport_) << std::endl;
+  if (verbose) {
+    std::cerr << "sending reply " << PacketToString(reply)
+              << " on port " << ((fd == udpfd_) ? udpport_ : tcpport_)
+              << ":" << getaddrport(addr) << std::endl;
+  }
 
   // Prefix with 2-byte length if TCP.
   if (fd != udpfd_) {
@@ -647,6 +819,88 @@ void MockChannelOptsTest::Process(unsigned int cancel_ms) {
               cancel_ms);
 }
 
+void MockEventThreadOptsTest::ProcessThread() {
+  std::set<ares_socket_t> fds;
+
+#ifndef CARES_SYMBOL_HIDING
+  bool has_cancel_ms = false;
+  struct timeval tv_begin;
+  struct timeval tv_cancel;
+#endif
+
+  mutex.lock();
+
+  while (isup) {
+    int nfds = 0;
+    fd_set readers;
+#ifndef CARES_SYMBOL_HIDING
+    struct timeval  tv_now = ares__tvnow();
+    struct timeval  tv_remaining;
+    if (cancel_ms_ && !has_cancel_ms) {
+      tv_begin  = ares__tvnow();
+      tv_cancel = tv_begin;
+      if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms_ << "ms" << std::endl;
+      tv_cancel.tv_sec  += (cancel_ms_ / 1000);
+      tv_cancel.tv_usec += ((cancel_ms_ % 1000) * 1000);
+      has_cancel_ms = true;
+    }
+#else
+    if (cancel_ms_) {
+      std::cerr << "library built with symbol hiding, can't test with cancel support" << std::endl;
+      return;
+    }
+#endif
+    struct timeval  tv;
+
+    /* c-ares is using its own event thread, so we only need to monitor the
+     * extrafds passed in */
+    FD_ZERO(&readers);
+    fds = MockEventThreadOptsTest::fds();
+    for (ares_socket_t fd : fds) {
+      FD_SET(fd, &readers);
+      if (fd >= (ares_socket_t)nfds) {
+        nfds = (int)fd + 1;
+      }
+    }
+
+#ifndef CARES_SYMBOL_HIDING
+    if (has_cancel_ms) {
+      unsigned int remaining_ms;
+      ares__timeval_remaining(&tv_remaining,
+                              &tv_now,
+                              &tv_cancel);
+      remaining_ms = (unsigned int)((tv_remaining.tv_sec * 1000) + (tv_remaining.tv_usec / 1000));
+      if (remaining_ms == 0) {
+        if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
+        ares_cancel(channel_);
+        cancel_ms_ = 0; /* Disable issuing cancel again */
+        has_cancel_ms = false;
+      }
+    }
+#endif
+
+    /* We just always wait 20ms then recheck. Not doing any complex signaling. */
+    tv.tv_sec  = 0;
+    tv.tv_usec = 20000;
+
+    mutex.unlock();
+    if (select(nfds, &readers, nullptr, nullptr, &tv) < 0) {
+      fprintf(stderr, "select() failed, errno %d\n", errno);
+      return;
+    }
+
+    // Let the provided callback process any activity on the extra FD.
+    for (ares_socket_t fd : fds) {
+      if (FD_ISSET(fd, &readers)) {
+        ProcessFD(fd);
+      }
+    }
+    mutex.lock();
+  }
+  mutex.unlock();
+
+}
+
 std::ostream& operator<<(std::ostream& os, const HostResult& result) {
   os << '{';
   if (result.done_) {
